@@ -51,7 +51,7 @@ struct tty_state {
 
      However, this only applies to cursor coordinates caused by text insertion.
      Inserting "X" at column 80 lets cursor.x go to 81; but if at column 70
-     and you do "move right by 20", the cursot ends up at 80, not 81.  Fun!
+     and you do "move right by 20", the cursor ends up at 80, not 81.  Fun!
 
      Details: https://github.com/mattiase/wraptest
    */
@@ -630,7 +630,7 @@ ansi_tty_print (ansi_tty *tty, unsigned long c)
   int scrolled_p = False;
 
   const char *kind = "?";
-  int av[255];
+  int av[255] = { UNDEF };
   int ac = 0;
   int i;
 
