@@ -1,4 +1,4 @@
-/* xscreensaver, Copyright (c) 2003-2017 Jamie Zawinski <jwz@jwz.org>
+/* xscreensaver, Copyright © 2003-2021 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -201,7 +201,9 @@ pick_font_1 (state *s, sentence *se)
 
 #  define _DONE_1 /**/
 #  define _DONE_2 /**/
+
 # if defined(HAVE_XFT) && !defined(HAVE_JWXYZ)   /* Real Xft under real X11 */
+
 #  undef  _DONE_1
 #  undef  _DONE_2
 #  define _DONE_1 DONE_1:
@@ -270,6 +272,7 @@ pick_font_1 (state *s, sentence *se)
       /* #### This gets a link error with FcFontSetDestroy missing. */
       /* if (fs) XftFontSetDestroy (fs); */
 
+      sprintf(pattern, "%s", name2);
       se->xftfont = font;
       ok = True;
       goto DONE_2;
@@ -455,8 +458,13 @@ pick_font_1 (state *s, sentence *se)
 
   if (! ok) return False;
 
-  se->xftfont = XftFontOpenXlfd (s->dpy, screen_number (s->xgwa.screen),
-                                 pattern);
+  /* Try pattern as an XLFD first, then if that fails, as Xft "Name Size". */
+  if (! se->xftfont)
+    se->xftfont = XftFontOpenXlfd (s->dpy, screen_number (s->xgwa.screen),
+                                   pattern);
+  if (! se->xftfont)
+    se->xftfont = load_xft_font_retry (s->dpy, screen_number (s->xgwa.screen),
+                                       pattern);
 
   _DONE_2
 
@@ -571,6 +579,22 @@ pick_font_1 (state *s, sentence *se)
       }
   }
 
+# if defined(HAVE_XFT) && !defined(HAVE_JWXYZ)   /* Real Xft under real X11 */
+  {
+    unsigned long uc = 0;
+    utf8_decode ((const unsigned char *) "M", 1, &uc);
+    if (!XftCharExists (s->dpy, se->xftfont, (FcChar32) uc))
+      {
+# ifdef DEBUG
+        if (s->debug_p)
+          fprintf (stderr, "%s: skipping font without ASCII: %s\n",
+                   progname, pattern2);
+# endif
+        return False;
+      }
+  }
+# endif
+
 
 # ifdef DEBUG
   if (s->debug_p) 
@@ -589,11 +613,15 @@ static void
 pick_font (state *s, sentence *se)
 {
   int i;
-  for (i = 0; i < 50; i++)
+  for (i = 0; i < (s->font_override ? 1 : 50); i++)
     if (pick_font_1 (s, se))
       return;
-  fprintf (stderr, "%s: too many font-loading failures: giving up!\n",
-           progname);
+  if (s->font_override)
+    fprintf (stderr, "%s: unable to load font \"%s\"\n",
+             progname, s->font_override);
+  else
+    fprintf (stderr, "%s: too many font-loading failures: giving up!\n",
+             progname);
   exit (1);
 }
 
@@ -681,6 +709,7 @@ get_word_text (state *s)
       int n = end - s->buf;
       memmove (s->buf, end, sizeof(s->buf) - n);
       s->buf_tail -= n;
+      if (s->buf_tail < 0) abort();
     }
 
   return result;
@@ -2266,6 +2295,7 @@ drain_input (state *s)
       else
         break;
     }
+  s->buf[s->buf_tail] = 0;
 }
 
 
