@@ -14,8 +14,6 @@
 
 #include <assert.h>
 
-#include "rotator.h"
-
 #define DEF_LIGHT     "True"
 #define DEF_FOG       "False"
 #define DEF_TEXTURE   "True"
@@ -60,27 +58,15 @@ enum {
 
 #define MARBLE_TEXTURE_SIZE 256
 
-#undef countof
-#define countof(x) (sizeof((x))/sizeof((*x)))
-
 #include <math.h>
 #include "xlockmore.h"
+#include "rotator.h"
+#include "doubletime.h"
 
 #ifdef USE_GL					/* whole file */
 
 typedef struct timeval glhtime;
 
-static double getTime(void)
-{
-	struct timeval t;
-#ifdef GETTIMEOFDAY_TWO_ARGS
-	gettimeofday(&t, NULL);
-#else							/* !GETTIMEOFDAY_TWO_ARGS */
-	gettimeofday(&t);
-#endif							/* !GETTIMEOFDAY_TWO_ARGS */
-	return t.tv_sec + t.tv_usec / 1000000.0;
-}
-
 typedef enum {
 	START,
 	MOVE_DISK,
@@ -617,7 +603,7 @@ static void enQTrail(glhcfg *glhanoi, GLfloat *posn)
 {	
 	if (glhanoi->trailQSize && glhanoi->state != MONEY_SHOT)  {
 		TrailPoint *tp = &(glhanoi->trailQ[glhanoi->trailQBack]);
-		double t = getTime();
+		double t = double_time();
 		
 		tp->position[0] = posn[0];
 		tp->position[1] = posn[1] + glhanoi->diskHeight;
@@ -705,7 +691,7 @@ static void updateView(glhcfg *glhanoi)
 static void changeState(glhcfg *glhanoi, State state)
 {
 	glhanoi->state = state;
-	glhanoi->startTime = getTime();
+	glhanoi->startTime = double_time();
 }
 
 static Bool finishedHanoi(glhcfg *glhanoi) {
@@ -717,7 +703,7 @@ static Bool finishedHanoi(glhcfg *glhanoi) {
 
 static void update_glhanoi(glhcfg *glhanoi)
 {
-	double t = getTime() - glhanoi->startTime;
+	double t = double_time() - glhanoi->startTime;
 	int i;
 	Bool done;
 
@@ -1424,13 +1410,14 @@ static GLubyte *makeTexture(glhcfg *glhanoi, int x_size, int y_size, int z_size,
 									   tex_col_t *), tex_col_t * colours)
 {
 	int i, j, k;
-	GLubyte *textureData;
+	GLuint *textureData;
 	GLuint *texturePtr;
 	double x, y, z;
 	double xi, yi, zi;
 
+	/* As we use GL_RGBA format, we must assign 4 bytes per element */
 	if((textureData =
-		calloc(x_size * y_size * z_size, sizeof(GLubyte))) == NULL) {
+		calloc(x_size * y_size * z_size, sizeof(*texturePtr))) == NULL) {
 		return NULL;
 	}
 
@@ -1439,7 +1426,7 @@ static GLubyte *makeTexture(glhcfg *glhanoi, int x_size, int y_size, int z_size,
 	zi = 1.0 / z_size;
 
 	z = 0.0;
-	texturePtr = (void *)textureData;
+	texturePtr = textureData;
 	for(k = 0; k < z_size; k++, z += zi) {
 		y = 0.0;
 		for(j = 0; j < y_size; j++, y += yi) {
@@ -1450,7 +1437,7 @@ static GLubyte *makeTexture(glhcfg *glhanoi, int x_size, int y_size, int z_size,
 			}
 		}
 	}
-	return textureData;
+	return (GLubyte *)textureData;
 }
 
 static void freeTexCols(tex_col_t*p)
@@ -1826,7 +1813,7 @@ static int drawTrails1(glhcfg *glhanoi, double t, double thickness, double alpha
 
 static int drawTrails(glhcfg *glhanoi) {
 	int lines = 0;
-	double t = getTime();
+	double t = double_time();
 
 	glEnable (GL_BLEND);
 	glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
@@ -1977,14 +1964,12 @@ ENTRYPOINT void draw_glhanoi(ModeInfo * mi)
 	update_glhanoi(glhanoi);
 	updateView(glhanoi);
 
-# ifdef HAVE_MOBILE	/* Keep it the same relative size when rotated. */
     {
-      GLfloat h = MI_HEIGHT(mi) / (GLfloat) MI_WIDTH(mi);
-      int o = (int) current_device_rotation();
-      if (o != 0 && o != 180 && o != -180)
-        glScalef (1/h, 1/h, 1/h);
+      GLfloat s = (MI_WIDTH(mi) < MI_HEIGHT(mi)
+                   ? (MI_WIDTH(mi) / (GLfloat) MI_HEIGHT(mi))
+                   : 1);
+      glScalef (s, s, s);
     }
-# endif
 
 	if(!glhanoi->wire && glhanoi->texture) {
 		glEnable(GL_TEXTURE_2D);
