@@ -1,5 +1,4 @@
-/* photopile, Copyright (c) 2008-2018 Jens Kilian <jjk@acm.org>
- * Based on carousel, Copyright (c) 2005-2008 Jamie Zawinski <jwz@jwz.org>
+/* photopile, Copyright © 2008-2025 Jamie Zawinski <jwz@jwz.org>
  * Loads a sequence of images and shuffles them into a pile.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -11,13 +10,8 @@
  * implied warranty.
  */
 
-#if defined(HAVE_COCOA) || defined(HAVE_ANDROID)
-# define DEF_FONT "OCR A Std 48, Lucida Console 48, Monaco 48"
-#elif 0  /* real X11, XQueryFont() */
-# define DEF_FONT "-*-helvetica-bold-r-normal-*-*-480-*-*-*-*-*-*"
-#else    /* real X11, load_font_retry() */
-# define DEF_FONT "-*-ocr a std-medium-r-*-*-*-480-*-*-m-*-*-*"
-#endif
+#define DEF_FONT \
+  "OCR A 18, OCR A Std 18, Lucida Console 18, Monaco 18, Courier 18, monospace 18"
 
 #define DEFAULTS  "*count:           7         \n" \
                   "*delay:           10000     \n" \
@@ -34,9 +28,6 @@
 # define release_photopile 0
 # define photopile_handle_event xlockmore_no_events
 
-#undef countof
-#define countof(x) (sizeof((x))/sizeof((*x)))
-
 #ifndef HAVE_JWXYZ
 # include <X11/Intrinsic.h>     /* for XrmDatabase in -debug mode */
 #endif
@@ -46,10 +37,11 @@
 #include "grab-ximage.h"
 #include "texfont.h"
 #include "dropshadow.h"
+#include "xftwrap.h"
 
 #ifdef USE_GL
 
-# define DEF_SCALE          "0.4"
+# define DEF_IMG_SCALE      "0.4"
 # define DEF_MAX_TILT       "50"
 # define DEF_SPEED          "1.0"
 # define DEF_DURATION       "5"
@@ -122,7 +114,8 @@ static Bool debug_p;         /* Be loud and do weird things. */
 
 
 static XrmOptionDescRec opts[] = {
-  {"-scale",        ".scale",         XrmoptionSepArg, 0 },
+  /* Xft uses 'scale' */
+  {"-scale",        ".imgScale",      XrmoptionSepArg, 0 },
   {"-maxTilt",      ".maxTilt",       XrmoptionSepArg, 0 },
   {"-speed",        ".speed",         XrmoptionSepArg, 0 },
   {"-duration",     ".duration",      XrmoptionSepArg, 0 },
@@ -141,7 +134,7 @@ static XrmOptionDescRec opts[] = {
 };
 
 static argtype vars[] = {
-  { &scale,         "scale",        "Scale",        DEF_SCALE,       t_Float},
+  { &scale,         "imgScale",     "Scale",        DEF_IMG_SCALE,   t_Float},
   { &max_tilt,      "maxTilt",      "MaxTilt",      DEF_MAX_TILT,    t_Float},
   { &speed,         "speed",        "Speed",        DEF_SPEED,       t_Float},
   { &duration,      "duration",     "Duration",     DEF_DURATION,    t_Int},
@@ -299,7 +292,7 @@ image_loaded_cb (const char *filename, XRectangle *geom,
     {
       /* strip filename to part after last /. */
       char *s = strrchr (frame->title, '/');
-      if (s) strcpy (frame->title, s+1);
+      if (s) memmove (frame->title, s+1, strlen (s));
     }
 
   if (debug_p)
@@ -447,6 +440,15 @@ reshape_photopile (ModeInfo *mi, int width, int height)
     if (o != 0 && o != 180 && o != -180)
       glScalef (1/h, h, 1);
   }
+# else
+  {
+    /* Don't understand why this clause doesn't work on mobile, but it 
+       doesn't. */
+    GLfloat s = (MI_WIDTH(mi) < MI_HEIGHT(mi)
+                 ? (MI_WIDTH(mi) / (GLfloat) MI_HEIGHT(mi))
+                 : 1);
+    glScalef (s, s, s);
+  }
 # endif
 
   glClear(GL_COLOR_BUFFER_BIT);
@@ -545,9 +547,9 @@ draw_image (ModeInfo *mi, int i, GLfloat t, GLfloat s, GLfloat z)
         }
       else
         {
-          GLfloat scale = minSize / maxSize;
-          w *= scale;
-          h *= scale;
+          GLfloat s2 = minSize / maxSize;
+          w *= s2;
+          h *= s2;
         }
 
       w1 = minSize * 1.16;      /* enlarge frame border */
@@ -649,72 +651,89 @@ draw_image (ModeInfo *mi, int i, GLfloat t, GLfloat s, GLfloat z)
     {
       int sw = 0, sh = 0;
       int ascent, descent;
-      GLfloat tw = w * 2;
-      GLfloat th = h1 - h;
-      GLfloat scale = 1;
-      const char *title = frame->title ? frame->title : "(untitled)";
+      char *title = frame->title ? frame->title : strdup ("(untitled)");
+      char *token, *line;
+      int lineno = 0;
       XCharStruct e;
+      GLfloat tpad = w * 0.05;
+      GLfloat tboxh = h1 - h - tpad * 2;
+      GLfloat tscale;
+      int nlines = 1;
 
       texture_string_metrics (ss->texfont, title, &e, &ascent, &descent);
       sw = e.width;
-      sh = ascent; /* + descent; */
+      sh = ascent + descent;
 
-      /* Scale the text to match the pixel size of the photo */
-      scale *= w / 150.0;
+      if (!polaroid_p)
+        tboxh = sh * 3;
 
-# if defined(HAVE_COCOA)
-      scale /= 2;
-      if (MI_WIDTH(mi) > 2560) scale /= 2;  /* Retina displays */
-# endif
+      glTranslatef (0, -(h + tpad), 0);
 
-# if defined(HAVE_MOBILE)
-      scale /= 2;
-# endif
-
-      /* Clip characters off the left end of the string until it fits. */
-      if (clip_p || polaroid_p)
-        while (sw * scale > tw && strlen (title) > 10)
-          {
-            title++;
-            texture_string_metrics (ss->texfont, title, &e, &ascent, &descent);
-            sw = e.width;
-          }
-
-      if (th <= 0)  /* Non-polaroid */
-        th = -sh * 1.2;
-
-      glTranslatef (-w, -h1, 0);
-      glTranslatef ((tw - sw*scale) / 2, (th - sh*scale) / 2, 0);
-
-      glScalef (scale, scale, 1);
+      /* Set the text height to fit 3 lines in the available space. */
+      tscale = tboxh / (sh * 3);
 
       if (wire || !polaroid_p)
-        {
-          glColor3f (1, 1, 1);
-        }
+        glColor3f (1, 1, 1);
       else
-        {
-          glColor3f (0.5, 0.5, 0.5);
-        }
+        glColor3f (0.5, 0.5, 0.5);
+
+      title = xft_word_wrap (MI_DISPLAY(mi), texfont_xft (ss->texfont),
+                             title, (w * 2) / tscale);
+      token = title;
+
+      glScalef (tscale, tscale, 1);
+      glTranslatef (0, -ascent, 0);
+
+      for (line = token; *line; line++)
+        if (*line == '\n') nlines++;
+
+      if (nlines == 1)
+        glTranslatef (0, -sh, 0);
+      else if (nlines == 2)
+        glTranslatef (0, -sh * 0.5, 0);
 
       if (!wire)
         {
           glEnable (GL_TEXTURE_2D);
           glEnable (GL_BLEND);
           glDisable (GL_DEPTH_TEST);
-          print_texture_string (ss->texfont, title);
-          glEnable (GL_DEPTH_TEST);
-        }
-      else
-        {
-          glBegin (GL_LINE_LOOP);
-          glVertex3f (0,  0,  0);
-          glVertex3f (sw, 0,  0);
-          glVertex3f (sw, sh, 0);
-          glVertex3f (0,  sh, 0);
-          glEnd();
         }
 
+      /* print_texture_string() does flushleft for newlines,
+         but we want the lines centered. */
+      while ((line = strtok (token, "\n")))
+        {
+          token = 0;
+          texture_string_metrics (ss->texfont, line, &e, 0, 0);
+          sw = e.width;
+
+          glPushMatrix();
+          glTranslatef (-sw/2, -sh * lineno, 0);
+
+          if (!wire)
+            {
+# ifndef HAVE_ANDROID   /* Doesn't work -- photo displays as static */
+              print_texture_string (ss->texfont, line);
+# endif
+            }
+          else
+            {
+              glBegin (GL_LINE_LOOP);
+              glVertex3f (0,  0,  0);
+              glVertex3f (sw, 0,  0);
+              glVertex3f (sw, sh, 0);
+              glVertex3f (0,  sh, 0);
+              glEnd();
+            }
+          glPopMatrix();
+          lineno++;
+
+          if (polaroid_p &&
+              (sh * (lineno + 1) - descent) * tscale > tboxh)
+            break;  /* Ran out of room for lines */
+        }
+
+      free (title);
     }
 
   glPopMatrix();
