@@ -71,8 +71,8 @@ static const char sccsid[] = "@(#)stairs.c	4.07 97/11/24 xlockmore";
 #if 0
 #include "e_textures.h"
 #else
-#include "xpm-ximage.h"
-#include "../images/wood.xpm"
+#include "ximage-loader.h"
+#include "images/gen/wood_png.h"
 #endif
 
 #include "sphere.h"
@@ -412,16 +412,11 @@ pinit(ModeInfo *mi)
     check_gl_error("mipmapping");
 #else
     {
-      XImage *img = xpm_to_ximage (mi->dpy,
-                                   mi->xgwa.visual,
-                                   mi->xgwa.colormap,
-                                   wood_texture);
+      XImage *img = image_data_to_ximage (mi->dpy, mi->xgwa.visual,
+                                          wood_png, sizeof(wood_png));
 	  glTexImage2D (GL_TEXTURE_2D, 0, GL_RGBA,
                     img->width, img->height, 0,
-                    GL_RGBA,
-                    /* GL_UNSIGNED_BYTE, */
-                    GL_UNSIGNED_INT_8_8_8_8_REV,
-                    img->data);
+                    GL_RGBA, GL_UNSIGNED_BYTE, img->data);
       check_gl_error("texture");
       XDestroyImage (img);
     }
@@ -477,7 +472,7 @@ draw_stairs (ModeInfo * mi)
 	if (!sp->glx_context)
 		return;
 
-	glXMakeCurrent(display, window, *(sp->glx_context));
+	glXMakeCurrent(display, window, *sp->glx_context);
 
 	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 
@@ -499,6 +494,16 @@ draw_stairs (ModeInfo * mi)
     if (rot != 0 && rot != 180 && rot != -180)
       glScalef (1/h, 1/h, 1/h);
   }
+# else
+  {
+    /* Don't understand why this clause doesn't work on mobile, but it 
+       doesn't. */
+    GLfloat s = (MI_WIDTH(mi) < MI_HEIGHT(mi)
+                 ? (MI_WIDTH(mi) / (GLfloat) MI_HEIGHT(mi))
+                 : 1);
+    glRotatef (current_device_rotation(), 0, 0, 1);
+    glScalef (s, s, s);
+  }
 # endif
 
     gltrackball_rotate (sp->trackball);
@@ -575,7 +580,7 @@ change_stairs (ModeInfo * mi)
 	if (!sp->glx_context)
 		return;
 
-	glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *(sp->glx_context));
+	glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *sp->glx_context);
 	pinit();
 }
 #endif /* !STANDALONE */
@@ -584,9 +589,10 @@ ENTRYPOINT void
 free_stairs (ModeInfo * mi)
 {
 	stairsstruct *sp = &stairs[MI_SCREEN(mi)];
-	if (glIsList(sp->objects)) {
-		glDeleteLists(sp->objects, 1);
-	}
+    if (!sp->glx_context) return;
+	glXMakeCurrent (MI_DISPLAY(mi), MI_WINDOW(mi), *sp->glx_context);
+	if (glIsList(sp->objects)) glDeleteLists(sp->objects, 1);
+    if (sp->trackball) gltrackball_free (sp->trackball);
 }
 
 XSCREENSAVER_MODULE ("Stairs", stairs)
