@@ -4,10 +4,10 @@ TARGET=$1
 shift
 SRCS=$*
 
-TMP1=m6502.h.1.$$
-TMP2=m6502.h.2.$$
+TMP1="m6502.h.1.$$"
+TMP2="m6502.h.2.$$"
 rm -f $TMP1 $TMP2
-trap "rm -f $TMP1 $TMP2" 1 2 3 15 EXIT
+trap 'rm -f $TMP1 $TMP2' 1 2 3 15 EXIT
 
 if [ -z "$UTILS_SRC" ]; then UTILS_SRC="../utils"; fi
 
@@ -17,10 +17,10 @@ for f in $SRCS ; do
     sed 's/",$/\\n"/' >> $TMP2
   echo ',' >> $TMP2
 done
-rm -f $TMP1
-if cmp -s $TMP2 $TARGET ; then
-  rm -f $TMP2
+rm -f "$TMP1"
+if cmp -s "$TMP2" "$TARGET" ; then
+  rm -f "$TMP2"
 else
-  mv $TMP2 $TARGET
+  mv "$TMP2" "$TARGET"
 fi
 
