@@ -1,4 +1,4 @@
-/* xscreensaver, Copyright (c) 1992-2021 Jamie Zawinski <jwz@jwz.org>
+/* xscreensaver, Copyright © 1992-2026 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -102,6 +102,12 @@
 # if defined(HAVE_COCOA) && !defined(HAVE_IPHONE)  /* macOS */
 #  include <OpenGL/gl.h>
 #  include <OpenGL/glu.h>
+#  define GL_DO_NOT_WARN_IF_MULTI_GL_VERSION_HEADERS_INCLUDED
+#  ifndef GL_SILENCE_DEPRECATION
+#   define GL_SILENCE_DEPRECATION
+#  endif
+#  include <OpenGL/gl3.h>
+#  undef GL_DO_NOT_WARN_IF_MULTI_GL_VERSION_HEADERS_INCLUDED
 # elif defined(HAVE_IPHONE)
 #  ifdef HAVE_GLES3
 #   include "OpenGLES/ES3/gl.h"
@@ -120,7 +126,15 @@
 #  else
 #   include <GL/glx.h>
 #  endif
-# endif
+
+# ifndef HAVE_JWZGLES
+   /* Sep 2022, Sep 2023: The Raspberry Pi 4b Broadcom driver doesn't do
+      GL_LINE_SMOOTH properly, so we must disable it. See init_GL(). */
+   extern void (* glEnable_fn) (GLuint);
+#  define glEnable (* glEnable_fn)
+# endif /* !HAVE_JWZGLES */
+
+# endif /* real X11 */
 
 # ifdef HAVE_JWZGLES
 #  include "jwzgles.h"
@@ -158,7 +172,7 @@
 #include "resources.h"
 #include "hsv.h"
 #include "colors.h"
-#include "grabscreen.h"
+#include "grabclient.h"
 #include "visual.h"
 #include "fps.h"
 #include "xft.h"
