@@ -1,4 +1,4 @@
-/* xscreensaver, Copyright (c) 1992-2014 Jamie Zawinski <jwz@jwz.org>
+/* xscreensaver, Copyright (c) 1992-2018 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -66,6 +66,10 @@ slidescreen_init (Display *dpy, Window window)
   st->grid_size = get_integer_resource (st->dpy, "gridSize", "Integer");
   st->pix_inc = get_integer_resource (st->dpy, "pixelIncrement", "Integer");
 
+  if (xgwa.width > 2560 || xgwa.height > 2560)
+    st->grid_size *= 2;  /* Retina displays */
+
+
   /* Don't let the grid be smaller than 5x5 */
   while (st->grid_size > xgwa.width / 5)
     st->grid_size /= 2;
@@ -88,6 +92,8 @@ slidescreen_init (Display *dpy, Window window)
       XParseColor (st->dpy, xgwa.colormap, "black", &bgc);
     if (!XParseColor (st->dpy, xgwa.colormap, bgs, &bgc))
       XParseColor (st->dpy, xgwa.colormap, "gray", &fgc);
+    if (fgs) free (fgs);
+    if (bgs) free (bgs);
 
     fg_ok = XAllocColor (st->dpy, xgwa.colormap, &fgc);
     bg_ok = XAllocColor (st->dpy, xgwa.colormap, &bgc);
@@ -172,10 +178,7 @@ slidescreen_init (Display *dpy, Window window)
 
   gcv.foreground = st->fg;
   gcv.function = GXcopy;
-  gcv.subwindow_mode = IncludeInferiors;
   gcflags = GCForeground |GCFunction;
-  if (use_subwindow_mode_p(xgwa.screen, st->window)) /* see grabscreen.c */
-    gcflags |= GCSubwindowMode;
   st->gc = XCreateGC (st->dpy, st->window, gcflags, &gcv);
 
   return st;
@@ -208,6 +211,9 @@ draw_grid (struct state *st)
       if (st->bitmap_h < st->grid_size*2) st->bitmap_h = st->grid_size*2;
     }
 
+  if (xgwa.width > 2560 || xgwa.height > 2560)
+    border *= 2;  /* Retina displays */
+
   st->grid_w = st->bitmap_w / st->grid_size;
   st->grid_h = st->bitmap_h / st->grid_size;
   st->hole_x = random () % st->grid_w;
@@ -472,11 +478,6 @@ slidescreen_free (Display *dpy, Window window, void *closure)
 static const char *slidescreen_defaults [] = {
   "*dontClearRoot:		True",
   "*fpsSolid:			true",
-
-#ifdef __sgi	/* really, HAVE_READ_DISPLAY_EXTENSION */
-  "*visualID:			Best",
-#endif
-
   ".background:			Black",
   ".foreground:			#BEBEBE",
   "*gridSize:			70",
